#!/bin/sh
#
# icpdconf.sh
#
# Interactive configuration script for ICP RAID Controller
# remote access service 'icpd'.
#
# Copyright (c) 2000-2001 ICP vortex Computersysteme GmbH, Neckarsulm
# All rights reserved.
#
# $Id: icpdconf.sh.redhat,v 1.3 2001/01/05 14:24:17 joerg Exp $

true=1
false=0

VERSION=1.05
ICPD_CONF=/etc/icpd.conf
TMP_PERM_FILE=/tmp/icpperm.tmp
TMP_PERM_FILE2=/tmp/icpperm2.tmp
TMP_TRGT_FILE=/tmp/icptrgt.tmp
TMP_EXEC_FILE=/tmp/icpexec.tmp
AWK=/usr/bin/awk
GREP=/bin/grep
SED=/usr/bin/sed
ICPDBIN=/usr/sbin/icpd
ICPDSCRIPT=icpd
RCSCRIPT=/etc/rc.d/init.d/$ICPDSCRIPT
RCSTARTLNK=/etc/rc.d/rc3.d/S99$ICPDSCRIPT
RCSTOPLNK=/etc/rc.d/rc3.d/K01$ICPDSCRIPT
ICPDINSTALL=$0

perm_count=0


rc_gen()
{
cat << EOF > $RCSCRIPT
#!/bin/bash
#
#
# Start ICP remote access service icpd.
#

# RedHat source function library
. /etc/rc.d/init.d/functions

ICPD=$ICPDBIN
RETVAL=0

case "\$1" in
    'start')
	if [ -x \$ICPD ]; then
		# start it.
		echo -n "Starting ICP RAID remote access service..."
		\$ICPD
		RETVAL=\$?
		if [ \$RETVAL -eq 0 ]; then
			echo "success"
		else
			echo "failed"
		fi
	else
		echo "Could not find ICP RAID remote access service ."
		RETVAL=1
	fi
	;;

    'stop')
	echo "Shutting down ICP RAID remote access service."
	killproc $ICPDBIN
	;;

    'restart'|'reload')
	\$0 stop
	\$0 start
	RETVAL=\$?
	;;

    'status')
	status icpd
	RETVAL=\$?
	;;

    *)
	echo "Usage: \$0 { start | stop | restart | reload | status }"
	exit 1
	;;
esac
exit \$RETVAL
EOF
chmod 755 $RCSCRIPT
if [ -L $RCSTARTLNK ]; then
	rm -f $RCSTARTLNK
fi
if [ -L $RCSTOPLNK ]; then
	rm -f $RCSTOPLNK
fi

ln -s ../init.d/$ICPDSCRIPT $RCSTARTLNK
ln -s ../init.d/$ICPDSCRIPT $RCSTOPLNK
}


#
# global ask function
#

ask()
{
	if [ -z "$2" ]; then
		echo -n "$1 "
	else
		echo -n "$1 [$2] "
	fi
	read answer
	if [ "$answer" = "Q" ]; then
		echo "Installation aborted."
		exit 1
	fi
	if [ -z "$answer" -a ! -z "$2" ]; then
		answer="$2"
	fi
}

#
# get current permissions from $ICPD_CONF
#
perm_get()
{
	if [ -r $TMP_PERM_FILE ]; then
		rm $TMP_PERM_FILE
	fi
	$AWK 'BEGIN{FS=" "; RS="\n"} $1 ~ /^user/ { print $0} $1 ~ /^group/ { print $0 }' $ICPD_CONF > $TMP_PERM_FILE
}

#
# get current SNMP targets and community from $ICPD_CONF
#
target_get()
{
	if [ -r $TMP_TRGT_FILE ]; then
		rm $TMP_TRGT_FILE
	fi
	$AWK 'BEGIN{FS=" "; RS="\n"} $1 ~ /^target/ { print $0} $1 ~ /^community/ { print $0 }' $ICPD_CONF > $TMP_TRGT_FILE
}

#
# get current execution parameters from $ICPD_CONF
#
execute_get()
{
	if [ -r $TMP_EXEC_FILE ]; then
		rm $TMP_EXEC_FILE
	fi
	$AWK 'BEGIN{FS=" "; RS="\n"} $1 ~ /^execute/ { print $0 }' $ICPD_CONF > $TMP_EXEC_FILE
}

#
# print temporary permissions
#
print_permissions()
{
	if [ -s $TMP_PERM_FILE ]; then
		$AWK 'BEGIN{i=1;FS="\n"} {print i":", $1; i++}' $TMP_PERM_FILE
	else
		echo "($ICPD_CONF does not exist or has no permissions defined)"
	fi
}

#
# ask for user name
#
perm_user()
{
user_ok=0
while [ $user_ok -eq 0 ]
do
	usergroup="user"
	echo ""
	ask "Enter user name or 'L' for Unix user list: " 
	if [ "$answer" = "" ]; then
		usergroup=""
		user_ok=1
	elif [ "$answer" = "L" ]; then
		echo ""
		$AWK 'BEGIN{FS=":"} {print "   ", $1}' /etc/passwd
	else
		dummy=`$GREP "^$answer:" /etc/passwd`
		if [ -z "$dummy" ]; then
			echo "user $answer does not exist in /etc/passwd! Please enter an existing user."
		elif [ ! -z "`$GREP \"^user $answer\" $TMP_PERM_FILE`" ]; then
			echo "Permission definition for user $answer already exists:"
			$GREP "^user $answer" $TMP_PERM_FILE
			echo "Please enter a user name, for which a permission is not yet defined."
		else
			name=$answer
			user_ok=1
		fi
	fi
done
}


#
# ask for group name
#
perm_group()
{
group_ok=0
while [ $group_ok -eq 0 ]
do
        usergroup="group"
	echo ""
        ask "Enter group name or 'L' for Unix group list: "
        if [ "$answer" = "" ]; then
                usergroup=""
                group_ok=1
        elif [ "$answer" = "L" ]; then
        	echo ""
                $AWK 'BEGIN{FS=":"} {print "    ",$1}' /etc/group
        else
                dummy=`$GREP "^$answer:" /etc/group`
                if [ -z "$dummy" ]; then
                        echo "Group $answer does not exist in /etc/group! Please enter an existing group."
                elif [ ! -z "`$GREP \"^group $answer\" $TMP_PERM_FILE`" ]; then
                        echo "Permission definition for group $answer already exists:"
                        $GREP "^group $answer" $TMP_PERM_FILE
                        echo "Please enter a group name, for which permission is not yet defined."
                else
                        name=$answer
                        group_ok=1
		fi
	fi
done
}


#
# define access type
#
perm_access()
{
acc_ok=0
while [ $acc_ok -eq 0 ]
do
	echo ""
	echo "Do you want to grant full (f) or readonly (r)"
	ask "remote access for $1? (f/r/Q)" r
	if [ "$answer" = "r" ]; then
		access="readonly"
		acc_ok=1
	elif [ "$answer" = "f" ]; then
		access="full"
		acc_ok=1
	fi
done
}


#
# add permission entry
#
perm_add()
{
add_ok=0
while [ $add_ok -eq 0 ]
do
	echo ""
	ask "User(u) or group(g) access permission? (u/g/Q)" u
	case "$answer" in
	    'u')
		perm_user
		add_ok=1
		;;
	    'g')
		perm_group
		add_ok=1
		;;
	    '')
		return 0
		;;
	esac
done

if [ -n "$usergroup" ]; then

	perm_access $name

	echo "$usergroup $name access $access" >> $TMP_PERM_FILE
	perm_count=$perm_count+1
fi
}

#
# delete permission entry
#
perm_delete()
{
        ask "Entry number to delete, or (q):" q
	if [ $answer = "q" ]; then
		echo "abort delete."
	else
		sed "${answer}d" $TMP_PERM_FILE > $TMP_PERM_FILE2 && cp $TMP_PERM_FILE2 $TMP_PERM_FILE
	fi
}

#
# interactive access configuration
#

access_configuration()
{
perm_get
# save SNMP targets/community
target_get
# save execution parameters
execute_get
ok=0
while [ $ok -eq 0 ]
do
	echo ""
        echo "Permissions configured so far:"
        echo ""
        print_permissions
	echo ""
	echo "Add (a), delete (d) a permission entry or"
	ask "continue (c) installation (a/d/c/Q)?" a

	case "$answer" in
	    'a')
		perm_add
		;;
	    'd')
		perm_delete
		;;
	    'c')
		ok=1
		;;
	esac
done

echo ""
ask "Save the configuration in $ICPD_CONF? (y/n/Q)" y
if [ "$answer" = "y" ]; then

echo ""
echo "Writing configuration to $ICPD_CONF..."
cat << EOF > $ICPD_CONF
# This is the configuration file for the
# ICP Disk Array Controller remote access daemon icpd.
# To enable the access you must have at least one entry
# in this file.
#
# Syntax: user | group <xxx> access <yyy>
# The user and/or group with name <xxx> must exist in /etc/passwd
# or /etc/group, respectively.
# Possible values for <yyy> are 'full' and 'readonly'.
#
# Example:
# user jack access readonly
# group icpadm access full
#
EOF
/bin/cat $TMP_PERM_FILE >> $ICPD_CONF
cat << EOF >> $ICPD_CONF
#
# icpd can send SNMP (v1) traps, if target hosts are defined
# like shown below. Default community is 'public'.
#
# Example:
# target 127.0.0.1
# target snmp.intranet.xyz.com
# community xyz
EOF
if [ -e $TMP_TRGT_FILE ]; then
	/bin/cat $TMP_TRGT_FILE >> $ICPD_CONF
	rm $TMP_TRGT_FILE
fi
cat << EOF >> $ICPD_CONF
#
#
#
# You can invoke an external command by 'execute' every time, when a
# controller event occurs. You can use following variables:
#   %H   Host name.
#   %C   Controller index (counting from 0)
#   %T   Time string.
#   %S   Severity (informational, warning, critical)
#   %E   Event string.
#   %I   Event ID.
# Use 'executeuser' to define the user to run the external command as.
#
# Example:
# executeuser nobody
# execute echo -e "Host      : %H\nController: %C\nTime      : %T\nSeverity  : %S\nEvent ID  : %I\n\nEvent text: %E" | mail -s "ICP: %S event" joerg@kermit
EOF
if [ -e $TMP_EXEC_FILE ]; then
	/bin/cat $TMP_EXEC_FILE >> $ICPD_CONF
	rm $TMP_EXEC_FILE
fi

echo "Configuration successfully saved."

else
	echo "Configuration not saved."
fi
}

#
# Ask for configuration
#

echo ""
echo "----------------------------------------------------------"
echo "| ICP Disk Array Controller Utility Configuration  v$VERSION |"
echo "----------------------------------------------------------"
echo ""
echo "This installation procedure guides you through the configuration"
echo "of the ICP Disk Array Controller remote access service (icpd). This"
echo "service is needed if you want to manage the ICP Disk Array controller"
echo "installed in this machine from a remote workstation."
echo ""
echo "You must have superuser rights to run this script successfully!"
echo ""
echo "Do you want to manage the ICP Disk Array controller installed"
ask  "in this machine from a remote workstation? (y/n)" y

if [ "$answer" = "n" -o "$answer" = "N" ]; then
	echo ""
	echo "Installation terminated by user."
	echo "This installation can be restarted from $ICPDINSTALL."
	echo ""
	exit 0
fi

echo ""
echo "You can abort the installation during input by typing 'Q' or"
echo "at any time by <ctrl> C."
echo ""
echo "Remote managability of the ICP Disk Array controller can be granted"
echo "only to already exisiting Unix users or groups. This installation"
echo "procedure does not create Unix users or groups."
echo ""
echo "Do you want to terminate the installation in order to create"
ask  "Unix users or groups now? (y/n)" n

if [ "$answer" = "y" -o "$answer" = "Y" ]; then
	echo ""
	echo "Installation terminated by user."
	echo "This installation can be restarted from $ICPDINSTALL."
	echo ""
	exit 0
fi

echo ""
ask "Do you want 'icpd' to be automatically started at startup? (y/n/Q)" y
automatic_start=$answer

if [ "$automatic_start" != "y" ]; then
        echo "No automatic start."
        if [ -L $RCSTARTLNK ]; then
           rm $RCSTARTLNK
        fi
        if [ -L $RCSTOPLNK ]; then
           rm $RCSTOPTLNK
        fi
else
	echo "Automatic start."
	rc_gen	
fi

#
# ask for manual or automatic permission definition
#
 
echo ""
echo "The ICP disk Array controller remote access service 'icpd' uses"
echo "the file $ICPD_CONF for its configuration at startup time."
echo "This file can now be created if it does not exist, or an existing file"
echo "can now be modified. All modifications are only written to $ICPD_CONF" 
echo "at the end of the installation procedure."
echo "$ICPD_CONF defines access permissions of users and groups which have"
echo "remote access to the ICP Disk Array controller installed in this machine."
echo "This access permissions are ICP specific, i.e. they do not affect the user or"
echo "group permissions for Unix in any way."
echo "The shadow passwd system is used for authentification."
echo ""
echo "Following access permissions can be granted:"
echo ""
echo "readonly      user or group can view controller configuration"
echo "full          user or group can view and change controller configuration"
echo ""

ok=0
interactive=0
while [ $ok -eq 0 ]
do
	ask "Do you want to create/modify $ICPD_CONF? (y/n/Q)" y
	if [ "$answer" = "y" ]; then
		ok=1
		interactive=1
	elif [ "$answer" = "n" -o "$answer" = "N" ]; then
		ok=1
		interactive=0
	fi
done

#
# interactive access configuration
#
if [ $interactive -eq 1 ]; then
	access_configuration
fi

#
# start service?
#
echo ""
ask "Do you want to start 'icpd' now? (y/n/Q)" y
if [ "$answer" = "y" ]; then
	# $RCSCRIPT stop
	$RCSCRIPT start
else
	echo "'icpd' not started."
fi

#
# finishing
#
echo ""
echo "Installation of 'icpd' finished successfully."
echo ""
